/***
 * -## Malato-Snake 2005 ##@
 *
 * Desenvolvido por:	Miguel Rosa Malato, Aluno do IST
 * Motivo:				Porque me apeteceu
 * Direitos:			Freeware
 * Agradecimentos:		Familia e Amigos
 *
 * @2005 MalatoSoft Corporation v1.0
 *
 ***/
#include "includes.h"

// Objectos Globais ao Jogo
Aplicacao *aplicacao;
Menus *menus;
Jogo *jogo;
HighScores *highScores;
//--

//// Funes do main
//
void setup(){
	// Inicializaes de dados
	aplicacao = new Aplicacao();
	menus = new Menus();
	jogo = new Jogo();
	highScores = new HighScores();
	// Inicializaes de OpenGL
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Cor usada quando se faz clear ao frameBuffer (Preto)
	GestorOpengl::disable(GL_DEPTH_TEST);
	GestorOpengl::disable(GL_LIGHTING);
	GestorOpengl::disable(GL_FOG);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1); // alinhamento dos pixeis. Assim podemos mostrar BMPs com kk tamanho (nao precisam ter largura multiplo de 4)
}

bool perguntaFullScreen() {
	int resposta;
	resposta = MessageBox(NULL, "Pretende jogar \"Malato-Snake\" em Full Screen ?", TITULO_JANELA, MB_ICONQUESTION | MB_YESNO);
	return resposta == IDYES;
}

//// CallBacks
//
void DisplayCB(){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// display do jogo
	switch(aplicacao->getEstadoApp()){
		case ESTADO_MENU_PRINCIPAL:
			menus->display_menuPrincipal();
			break;
		case ESTADO_JOGO:
			jogo->display_jogo();
			break;
		case ESTADO_GAMEOVER:
			menus->display_gameOver();
			break;
		case ESTADO_HIGHSCORES:
			menus->display_highScores();
			break;
		case ESTADO_AJUDA:
			menus->display_ajuda();
			break;
		case ESTADO_CREDITOS:
			menus->display_creditos();
			break;
		default:
			break;
	}
	//---
	glutSwapBuffers();
	glFlush();
}

void ReshapeCB(int w, int h){
	// nao permite reshapes de janela
	if((w != VIEWPORT_W) || (h != VIEWPORT_H)){
		glutReshapeWindow(VIEWPORT_W, VIEWPORT_H);
		return;
	}

	// actualiza o novo tamanho da janela no objecto aplicacao
	aplicacao->setWindowWidth(w);
	aplicacao->setWindowHeight(h);

	glViewport(0, 0, w, h);	// Ajusta o Viewport para as dimensoes da janela

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0, w, 0, h);	// sistema de Coordenadas 2D
	glScalef(1, -1, 1);		// inverte o eixo dos yy. Para baixo  positivo
	glTranslatef(0, -h, 0);	// origem do referencial no canto superior esquerdo

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	// redesenha a cena
	glutPostRedisplay();
}

void KeyboardCB(unsigned char key, int x, int y){
	switch(aplicacao->getEstadoApp()){
		case ESTADO_MENU_PRINCIPAL:
			menus->interacao_menuPrincipal(key, -1, x, y);
			break;
		case ESTADO_JOGO:
			jogo->interacao_jogo(key, -1, x, y);
			break;
		case ESTADO_GAMEOVER:
			menus->interacao_gameOver(key, -1, x, y);
			break;
		case ESTADO_HIGHSCORES:
			menus->interacao_highScores(key, -1, x, y);
			break;
		case ESTADO_AJUDA:
			menus->interacao_ajuda(key, -1, x, y);
			break;
		case ESTADO_CREDITOS:
			menus->interacao_creditos(key, -1, x, y);
			break;
		default:
			break;
	}
}

void SpecialKeyboardCB(int key, int x, int y){
	switch(aplicacao->getEstadoApp()){
		case ESTADO_MENU_PRINCIPAL:
			menus->interacao_menuPrincipal('0', key, x, y);
			break;
		case ESTADO_JOGO:
			jogo->interacao_jogo('0', key, x, y);
			break;
		case ESTADO_GAMEOVER:
			menus->interacao_gameOver('0', key, x, y);
			break;
		case ESTADO_HIGHSCORES:
			menus->interacao_highScores('0', key, x, y);
			break;
		case ESTADO_AJUDA:
			menus->interacao_ajuda('0', key, x, y);
			break;
		case ESTADO_CREDITOS:
			menus->interacao_creditos('0', key, x, y);
			break;
		default:
			break;
	}
}

void UpdateJogoCB(int value) {
	if(aplicacao->getEstadoApp() == ESTADO_JOGO){
		// actualiza o estado interno do jogo
		jogo->update_jogo();
		// Redesenha a cena
		glutPostRedisplay();
	}
	glutTimerFunc(1000/aplicacao->fps, UpdateJogoCB, 1); // Frames por segundo
}

//// Main da Aplicao
//
void main(){
	// Definies para a janela
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_ACCUM);
	glutInitWindowSize(VIEWPORT_W, VIEWPORT_H);
	glutInitWindowPosition(WINDOW_POSITION_X, WINDOW_POSITION_Y);
	// FullScreen VS Janela
	if(perguntaFullScreen()){
		glutGameModeString(DEFINICOES_FULL_SCREEN);
		if(glutGameModeGet(GLUT_GAME_MODE_POSSIBLE)){
			glutEnterGameMode(); // activa o modo FullScreen
			ShowCursor(false);	 // esconde o ponteiro do rato
		}
		else glutCreateWindow(TITULO_JANELA);
	} else glutCreateWindow(TITULO_JANELA);
	// Registo das CallBacks
	glutDisplayFunc(DisplayCB);
	glutReshapeFunc(ReshapeCB);
	glutKeyboardFunc(KeyboardCB);
	glutSpecialFunc(SpecialKeyboardCB);
	glutTimerFunc(33, UpdateJogoCB, 1);
	glutIgnoreKeyRepeat(1);
	// Inicializaes de dados e OpenGL
	setup();
	// Processamento de Eventos
	glutMainLoop();
}

// Main para uma Win32 Aplication
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd){
	main();
	return 0;
}